/*
    S60Bible A Bible Reader for SymbianOS
    Copyright (C) 2004 Yohanes Nugroho

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Yohanes Nugroho (yohanes@opensource.or.id)
    Kp Areman RT 09/08 No 71 Desa Tugu Cimanggis Depok 16951 Indonesia
*/


/*
 * Filename: S60BibleAppView.cpp
 * Description: Bible view class
 * Author: Yohanes Nugroho
 * Creation Date:
 * Last Update: 15/Feb/2004
 * Revisions:
 */

#include <coemain.h>
#include <eikmenub.h>
#include <eikenv.h>
#include <e32keys.h>
#include <aknutils.h>
#include <e32std.h>
#include <aknnotewrappers.h>
#include <aknviewappui.h>
#include <eiktxlbx.h>
#include <aknpopup.h>
#include <aknlists.h>
#include <baclipb.h>
#include <hal.h>
#include <bautils.h>
#include <charconv.h>
#include <aknselectionlist.h>

#include "S60BibleMainView.h"
#include "S60BibleDocument.h"
#include "S60BibleUid.h"
#include "S60Bible.hrh"
#include "S60Bible.pan"
#include <S60Bible.rsg>
#include "util.h"
#include "log.h"
#include "CStringList.h"
#include "ColorDialog.h"

#ifdef __SERIES60_30__
#include "CheckBiblePDB.h"
#endif

CS60BibleMainView* CS60BibleMainView::NewL(const TRect& aRect, CS60BibleDocument* aDocument)
{
	CS60BibleMainView* self = CS60BibleMainView::NewLC(aRect, aDocument);
	CleanupStack::Pop(self);
	return self;
}

CS60BibleMainView* CS60BibleMainView::NewLC(const TRect& aRect, CS60BibleDocument* aDocument)
{
	CS60BibleMainView* self = new (ELeave) CS60BibleMainView();
	CleanupStack::PushL(self);
	self->iDocument = aDocument;
	self->ConstructL(aRect);
	return self;
}


TUid CS60BibleMainView::Id() const
{
	return TUid::Uid(KMainViewID);
}


void CS60BibleMainView::LostFocus()
{
	if (iBackLight && iContainer) {
		iContainer->SetBackLight(EFalse);
	}
}

void CS60BibleMainView::GainFocus()
{
	if (iBackLight && iContainer) {
		iContainer->SetBackLight(ETrue);
	}
}

CS60BibleMainView::CS60BibleMainView()
{

}


CS60BibleMainView::~CS60BibleMainView()
{
    if (iOffset)
        iCoeEnv->DeleteResourceFile(iOffset);

	delete iHistory;
}

void CS60BibleMainView::LoadSettings(TBool aFirst)
{
	CStringList *settings = CStringList::NewLC(CEikonEnv::Static()->FsSession());
	settings->LoadFromFileL(iBibleOptions);
	if (settings->Count()>1){
		if (aFirst) iFullScreen = settings->GetNumberAt(0);
		iBackLight = settings->GetNumberAt(1);
	}
	if (settings->Count()>2){
		iFullBookName = settings->GetNumberAt(2);		
	}
	
	CleanupStack::PopAndDestroy(settings);
}


void CS60BibleMainView::LoadFontsL(const TDesC& aExtension)
{
	TFileName file;
#ifdef __SERIES60_30__
	_LIT(KFontDir, "c:\\data\\fonts\\");
	file.Copy(KFontDir);	
#else	
	HBufC* appName = iDocument->Application()->AppFullName().AllocLC();                    
    TParsePtrC parse(*appName);                                                                                                              
    // Construct search path
    file.Copy(parse.DriveAndPath());
#endif     
    file.Append(aExtension);                                                        

	Log(_L("Searching"));
	Log(file);
    RFs& fs = iCoeEnv->FsSession();                                             
                                                                                
    CDir* entries;                                                              
    fs.GetDir(file, KEntryAttNormal, ESortByName, entries);                     
    if (entries!=NULL) {
    CleanupStack::PushL(entries); 
    for (TInt i = 0; i < entries->Count(); ++i)                                 
	{
		TInt id = 0;
#ifdef __SERIES60_30__
		tmpFontPath.Copy(KFontDir);
#else		
		tmpFontPath.Copy(parse.DriveAndPath());
#endif
		tmpFontPath.Append((*entries)[i].iName);
		
		CEikonEnv::Static()->ScreenDevice()->AddFile(tmpFontPath, id);
	}

	CleanupStack::PopAndDestroy(entries);
    }
#ifndef __SERIES60_30__
	CleanupStack::PopAndDestroy(appName);
#endif
}

void CS60BibleMainView::ConstructL(const TRect& /*aRect*/)
{

	BaseConstructL(R_S60BIBLE_MAINVIEW);
	_LIT(KGDR, "*.gdr");
	LoadFontsL(KGDR);
	_LIT(KTTF, "*.ttf");
	LoadFontsL(KTTF);
	_LIT(KCCC, "*.ccc");
	LoadFontsL(KCCC);

	HBufC *appDll = HBufC::NewLC(KMaxFileName);

	*appDll = iDocument->Application()->AppFullName();
	TParse parsa;
	User::LeaveIfError(parsa.SetNoWild(*appDll, NULL, NULL));

#ifdef __SERIES60_30__

	//iDataPath = _L("\\private\\A0201077\\");
	CEikonEnv::Static()->FsSession().PrivatePath(iDataPath);
#else	
	iDataPath = parsa.DriveAndPath();
	// If the application is in ROM (drive Z:), put settings on C:
	if ( (iDataPath[0] == 'Z') || (iDataPath[0] == 'z') ) {
		iDataPath[0] = 'C';
	}
#endif
	iCrashFile = iDataPath;
	iCrashFile.Append(KCrashFile);
	

	iBibleList = iDataPath;
	iBibleList.Append(KBibleListFile);

	iBibleSettings = iDataPath;
	iBibleSettings.Append(KSettingsFile);

	iBibleOptions = iDataPath;
	iBibleOptions.Append(KOptionsFile);

	/*the crash file should not exist*/
	if (BaflUtils::FileExists(CEikonEnv::Static()->FsSession(), iCrashFile)) {
		/*remove settings file to prevent loading same bible file*/
		BaflUtils::DeleteFile(CEikonEnv::Static()->FsSession(), iBibleSettings);		
	}


	iDocument->Model()->iOptionsFile = iBibleOptions;

	iDocument->Model()->iBookmarkFile = iDataPath;
	iDocument->Model()->iBookmarkFile.Append(KBookmarkFile);

	iDocument->Model()->iBindingsFile = iDataPath;
	iDocument->Model()->iBindingsFile.Append(KBindingsFile);

	iBibleHistory = iDataPath;
	iBibleHistory.Append(KHistoryFile);

	CStringList *settings = CStringList::NewLC(CEikonEnv::Static()->FsSession());
	settings->LoadFromFileL(iBibleSettings);

	iHistory = CStringList::NewL(CEikonEnv::Static()->FsSession());
	iHistory->LoadFromFileL(iBibleHistory);

	int book = 10;
	int chapter = 1;
	int verse = 1;
	
//#define _DEBUG_
#ifdef _DEBUG_
	BaflUtils::CopyFile(CEikonEnv::Static()->FsSession(), iBibleSettings, _L("C:\\data\\biblesettings.txt"));
	BaflUtils::CopyFile(CEikonEnv::Static()->FsSession(), iBibleOptions, _L("C:\\data\\bibleoptions.txt"));
#endif

	
	//ugly, should have used streams instead
	//but sometimes it is usefull to be able to quickly see the content 
	//of the configuration file
	if (settings->Count()){
		TPtrC  str = (*(settings->Content()))[0];
		if (str.Length()>0) {
			CStringList *crashnote = CStringList::NewLC(CEikonEnv::Static()->FsSession());
			crashnote->AddStringL(str);
			crashnote->SaveToFileL(iCrashFile);
			CleanupStack::PopAndDestroy(crashnote);
			
			if (iDocument->Model()->ChangeBible(str)) {
				Log(_L("Load OK"));
				if (settings->Count()>5) {
					iLastVerse = (*(settings->Content()))[1];
					book = settings->GetNumberAt(2);
					chapter = settings->GetNumberAt(3);
					verse = settings->GetNumberAt(4);
					iDocument->Model()->setFont(settings->GetNumberAt(5));
					iDocument->Model()->SetEncoding(settings->GetNumberAt(6));
				}			
				if (settings->Count()>12) {
					for (int i = 0; i<6; i++){
						TRgb color(settings->GetNumberAt(7+i));
						iDocument->Model()->SetColor((Viewer::ETextObject)i, color);
					}
				}
				if (settings->Count()>13) {
					iDocument->Model()->SetZoomLevel(settings->GetNumberAt(13));
				}
				iDocument->Model()->ClearVerse();
				if (book<0) book = 10;
				if (chapter<0) chapter = 1;
				if (verse<0) verse = 1;
				iDocument->Model()->SeekVerse(1, (UInt16) book, (UInt16)chapter,(UInt16) verse);
				iDocument->Model()->FillVerse();
			}
		}
	}
	CleanupStack::PopAndDestroy(settings);
	LoadSettings(ETrue);
	iDocument->Model()->SetLongBookName(iFullBookName);
	CleanupStack::PopAndDestroy(appDll); //hbufc
	BaflUtils::DeleteFile(CEikonEnv::Static()->FsSession(), iCrashFile);		

}

void CS60BibleMainView::DoActivateL(const TVwsViewId& aPrevViewId,
                                    TUid aCustomMessageId,
                                    const TDesC8& /*aCustomMessage*/)
{
	LoadSettings(EFalse);
	ASSERT(iContainer == NULL);
	CEikButtonGroupContainer::Current()->MakeVisible(ETrue);
	iContainer = CS60BibleMainContainer::NewL(this, iDocument);
	AppUi()->AddToStackL(iContainer);
	iDocument->Model()->LoadBookmarkL();
	iDocument->Model()->LoadBindings();
	iContainer->SetFullScreenL(iFullScreen);
	iContainer->SetBackLight(iBackLight);
	iDocument->Model()->SetLongBookName(iFullBookName);		
	if (aPrevViewId.iViewUid==TUid::Uid(KBookListViewId)) {
		if (aCustomMessageId==TUid::Uid(1)) { /*a book is selected*/
				/*display chapter and verse dialog*/
				TInt chapter(1);
				CAknNumberQueryDialog *dlg = new (ELeave)CAknNumberQueryDialog(chapter);
				dlg->PrepareLC(R_CHAPTER_QUERY);
				int max = iDocument->Model()->GetTotalBookChapter((UInt16)iDocument->Model()->GetBookNumber());
				//LogL(_L("Max chapter"), max);
				dlg->SetMinimumAndMaximum(1, max);
				if (!dlg->RunLD()) {
					iDocument->Model()->ClearVerseList();	
					iDocument->Model()->FillVerse();							
					return;	
				}
				TInt verse(1);
				CAknNumberQueryDialog *dlg2 = new (ELeave)CAknNumberQueryDialog(verse);
				dlg2->PrepareLC(R_VERSE_QUERY);
				max = iDocument->Model()->GetTotalChapterVerse((UInt16)iDocument->Model()->GetBookNumber(),
					chapter);
				//LogL(_L("Max verse"), max);
				dlg2->SetMinimumAndMaximum(1, max);
				dlg2->RunLD();
				iDocument->Model()->ClearVerse();
				//LogL(_L("chapter"), chapter);
				//LogL(_L("verse"), verse);
				iDocument->Model()->SeekVerse(1, (UInt16)iDocument->Model()->GetBookNumber(), 
					(UInt16)chapter,(UInt16) verse);
				//LogL(_L("Seek done"));
				iDocument->Model()->FillVerse();
				//LogL(_L("Fill done"));
		}
	}  else {
		iDocument->Model()->ClearVerseList();	
		iDocument->Model()->FillVerse();		
	}
	
}

void CS60BibleMainView::DoDeactivate()
{
	if (iContainer)	{
		AppUi()->RemoveFromStack(iContainer);
		delete iContainer;
		iContainer = NULL;
	}
}


void CS60BibleMainView::ShowHistory()
{
	CEikTextListBox* list = new(ELeave) CAknSinglePopupMenuStyleListBox;
	CleanupStack::PushL(list);
	CStringList *strlist = CStringList::NewLC(CEikonEnv::Static()->FsSession());

	// Create popup list and PUSH it.
	CAknPopupList* popupList = CAknPopupList::NewL(
					list, R_AVKON_SOFTKEYS_OK_BACK,
					AknPopupLayouts::EMenuWindow);
	CleanupStack::PushL(popupList);

	// initialize listbox.
	list->ConstructL(popupList, CEikListBox::ELeftDownInViewRect);
	list->CreateScrollBarFrameL(ETrue);
	list->ScrollBarFrame()->SetScrollBarVisibilityL(
				CEikScrollBarFrame::EOff,
				CEikScrollBarFrame::EAuto);

	// Set listitems.
	CTextListBoxModel* model = list->Model();
	strlist->Copy(iHistory);

	model->SetItemTextArray(strlist->Content());

	model->SetOwnershipType(ELbmDoesNotOwnItemArray);

	HBufC* title = CCoeEnv::Static()->AllocReadResourceLC(R_S60BIBLE_STRING_HISTORY);

	//_LIT(title, "History");
	popupList->SetTitleL(*title);

	// Show popup list and then show return value.
	TInt popupOk = popupList->ExecuteLD();

	CleanupStack::PopAndDestroy(title);
	CleanupStack::Pop(popupList);
	if (popupOk){
		TInt index = list->CurrentItemIndex();
		CDesCArrayFlat *tmp = strlist->Content();
		char buff[256];
		iDocument->Model()->MyDesToString((*tmp)[index], buff);
		if (iDocument->Model()->SeekVerseAsString(1, buff)) {
				iDocument->Model()->ClearVerse();
				iDocument->Model()->FillVerse();
		}
	}
	CleanupStack::PopAndDestroy(strlist);
	CleanupStack::PopAndDestroy(list);
}

void CS60BibleMainView::FontNumToName(TDes& aFontName, const TInt aFontNum) {
	CWsScreenDevice* dev = iCoeEnv->ScreenDevice();
	TTypefaceSupport typefaceSupport;
	dev->TypefaceSupport(typefaceSupport, aFontNum);
	aFontName.Copy(typefaceSupport.iTypeface.iName);
}

TInt CS60BibleMainView::FontNameToNum(TDesC& aFontName) {
	CWsScreenDevice* dev = iCoeEnv->ScreenDevice();
	for ( TInt t = 0; t < dev->NumTypefaces(); t++ ) {
		TTypefaceSupport typefaceSupport;
		dev->TypefaceSupport(typefaceSupport, t);
		if( aFontName.CompareF( typefaceSupport.iTypeface.iName.Des() ) == 0 ) {
			return t;
		}
	}
	return 0;
}

void CS60BibleMainView::ShowFontSelection() {
        CEikTextListBox* list = new(ELeave) CAknSinglePopupMenuStyleListBox;
        CleanupStack::PushL(list);
        CAknPopupList* popupList = CAknPopupList::NewL(
                                        list, R_AVKON_SOFTKEYS_OK_BACK,
                                        AknPopupLayouts::EMenuWindow);
        CleanupStack::PushL(popupList);

	CWsScreenDevice* dev = iCoeEnv->ScreenDevice();
        CDesCArrayFlat * selection = new (ELeave) CDesCArrayFlat(2);

	for ( TInt t = 0; t < dev->NumTypefaces(); t++ ) {
		TTypefaceSupport typefaceSupport;
		dev->TypefaceSupport(typefaceSupport, t);
		selection->AppendL(typefaceSupport.iTypeface.iName.Des());
	}
        list->ConstructL(popupList, CEikListBox::ELeftDownInViewRect);
        list->CreateScrollBarFrameL(ETrue);
        list->ScrollBarFrame()->SetScrollBarVisibilityL(
                                CEikScrollBarFrame::EOff,
                                CEikScrollBarFrame::EAuto);

        // Set listitems.
        CTextListBoxModel* model = list->Model();
        model->SetItemTextArray(selection);
        model->SetOwnershipType(ELbmOwnsItemArray);
        TInt popupOk = popupList->ExecuteLD();
        //do selection here
	if (popupOk) {
		TInt index = list->CurrentItemIndex();
		iDocument->Model()->setFont(index);
		iDocument->Model()->FillVerse();
	}
        CleanupStack::Pop();            // popuplist
        CleanupStack::PopAndDestroy(list);  // list
}




void CS60BibleMainView::ShowEncodingSelection()
{
        // Create listbox and PUSH it.
        CEikTextListBox* list = new(ELeave) CAknSinglePopupMenuStyleListBox;
        CleanupStack::PushL(list);

        CCnvCharacterSetConverter* characterSetConverter=CCnvCharacterSetConverter::NewLC();
        // Create popup list and PUSH it.
        CAknPopupList* popupList = CAknPopupList::NewL(
                                        list, R_AVKON_SOFTKEYS_OK_BACK,
                                        AknPopupLayouts::EMenuWindow);
        CleanupStack::PushL(popupList);
        RFs fs;
        fs = CEikonEnv::Static()->FsSession();
        //fs.Connect();
        CArrayFix<CCnvCharacterSetConverter::SCharacterSet> *charsets =
          characterSetConverter->CreateArrayOfCharacterSetsAvailableL(
            fs);

        CDesCArrayFlat * selection = new CDesCArrayFlat(2);

        for (TInt i=charsets->Count()-1; i>=0; --i) {
            TParse parsa;

              const CCnvCharacterSetConverter::SCharacterSet& charactersSet=
                                  (*charsets)[i];
              //characterSetConverter->PrepareToConvertToOrFromL(charactersSet.Identifier(),
               //            *charsets, fileServerSession);
               TPtrC charactersSetName=charactersSet.Name();
              //Console.Printf(KFormatting, &charactersSetName);
              User::LeaveIfError(parsa.SetNoWild(charactersSetName, NULL, NULL));
              selection->AppendL(parsa.Name());
        }
        list->ConstructL(popupList, CEikListBox::ELeftDownInViewRect);
        list->CreateScrollBarFrameL(ETrue);
        list->ScrollBarFrame()->SetScrollBarVisibilityL(
                                CEikScrollBarFrame::EOff,
                                CEikScrollBarFrame::EAuto);

        // Set listitems.
        CTextListBoxModel* model = list->Model();
        model->SetItemTextArray(selection);
        model->SetOwnershipType(ELbmOwnsItemArray);
        TInt popupOk = popupList->ExecuteLD();

	if (popupOk) {
		TInt index = list->CurrentItemIndex();
		const CCnvCharacterSetConverter::SCharacterSet& charactersSet=
                                  (*charsets)[index];
		iDocument->Model()->SetEncoding(charactersSet.Identifier());
		iDocument->Model()->FillVerse();
	}
        delete charsets;

        CleanupStack::Pop();            // popuplist
        CleanupStack::PopAndDestroy(characterSetConverter);
        CleanupStack::PopAndDestroy(list);  // list
}

void CS60BibleMainView::ShowBibleSelection()
{
	// Create listbox and PUSH it.
	CEikTextListBox* list = new(ELeave) CAknSinglePopupMenuStyleListBox;
	CleanupStack::PushL(list);
	// Create popup list and PUSH it.
	CAknPopupList* popupList = CAknPopupList::NewL(
					list, R_AVKON_SOFTKEYS_OK_BACK,
					AknPopupLayouts::EMenuWindow);
	CleanupStack::PushL(popupList);

	// initialize listbox.
	list->ConstructL(popupList, CEikListBox::ELeftDownInViewRect);
	list->CreateScrollBarFrameL(ETrue);
	list->ScrollBarFrame()->SetScrollBarVisibilityL(
				CEikScrollBarFrame::EOff,
				CEikScrollBarFrame::EAuto);
	CStringList *strlist = CStringList::NewLC(CEikonEnv::Static()->FsSession());
	strlist->LoadFromFileL(iBibleList);

	CDesCArrayFlat * selection = new CDesCArrayFlat(2);
	for(int i=0; i<strlist->Content()->Count(); i++) {
		TParse parsa;
		User::LeaveIfError(parsa.SetNoWild((*(strlist->Content()))[i], NULL, NULL));
		selection->AppendL(parsa.Name());
	}

	// Set listitems.
	CTextListBoxModel* model = list->Model();

	model->SetItemTextArray(selection);
	model->SetOwnershipType(ELbmOwnsItemArray);
	//model->SetOwnershipType(ELbmDoesNotOwnItemArray);
	//CleanupStack::Pop();

	HBufC* title = CCoeEnv::Static()->AllocReadResourceLC(R_S60BIBLE_STRING_SELECT_BIBLE);

	//_LIT(title, "Select Bible");
	popupList->SetTitleL(*title);

	// Show popup list and then show return value.
	TInt popupOk = popupList->ExecuteLD();
	CleanupStack::PopAndDestroy(title);
	CleanupStack::Pop();            // strlist
        CleanupStack::Pop();            // popuplist
        CleanupStack::PushL(strlist);
	if (popupOk){

		TInt index = list->CurrentItemIndex();
		CDesCArrayFlat *tmp = strlist->Content();
		if (iDocument->Model()->ChangeBible((*tmp)[index])) {
                        iDocument->Model()->FillVerse();
			CStringList *settings = CStringList::NewLC(CEikonEnv::Static()->FsSession());
			settings->AddStringL((*tmp)[index]);
			settings->AddStringL(iLastVerse);
			settings->SaveToFileL(iBibleSettings);
			CleanupStack::PopAndDestroy(settings);
		}
	}
	CleanupStack::PopAndDestroy(strlist);
	CleanupStack::PopAndDestroy(list);  // list
}

void CS60BibleMainView::SaveSettingsL()
{
		CStringList *settings = CStringList::NewLC(
			CEikonEnv::Static()->FsSession());
		settings->AddStringL(iDocument->Model()->iCurrentBook);
		settings->AddStringL(iLastVerse);
		settings->AddNumberL(iDocument->Model()->GetBookNumber());
		settings->AddNumberL(iDocument->Model()->GetStartChapter());
		settings->AddNumberL(iDocument->Model()->GetStartVerse());
		settings->AddNumberL(iDocument->Model()->GetFontNum());
		settings->AddNumberL(iDocument->Model()->GetEncoding());
		for (int i = 0; i<6; i++){
			TRgb color = iDocument->Model()->GetColor((Viewer::ETextObject)i);
			settings->AddNumberL(color.Value());
		}		
		settings->AddNumberL(iDocument->Model()->GetZoomLevel());		
		Log(_L("Saving to"));
		Log(iBibleSettings);

		//CEikonEnv::Static()->FsSession().CreatePrivatePath(EDriveA);

		settings->SaveToFileL(iBibleSettings);
		CleanupStack::PopAndDestroy(settings);
		Log(_L("Saving history to"));
		Log(iBibleHistory);		
		iHistory->SaveToFileL(iBibleHistory);
}

#if 0

TInt CS60BibleMainView::CycleError  (  TInt  aError   )
{
	return KErrNone;	
}


void CS60BibleMainView::DialogDismissedL  (  TInt    )
{
	iFinderEngine->CancelFind(); 
}



TBool CS60BibleMainView::IsProcessDone  () const
{
	return iFinderEngine->IsDone();
}

void CS60BibleMainView::ProcessFinished  ()
{
	
}

void CS60BibleMainView::StepL  ()
{
	iFinderEngine->FindStep();
}
#endif

void  CS60BibleMainView::DialogDismissedL (  TInt/*    aButtonId */ )
{
	iSearchDone = ETrue;
}



void CS60BibleMainView::SearchForBibleFilesL()
{
	CAknWaitDialog *dlg = new (ELeave) CAknWaitDialog(NULL, ETrue);
	dlg->SetCallback(this);
	/* despite of the name (ends with a D),
	 *  the docs says that this doesn't delete the dialog
	 */
	dlg->ExecuteLD(R_FIND);

	iSearchDone = EFalse;
	CStringList *strlist =CStringList::NewLC(CEikonEnv::Static()->FsSession());
	CFileFinder *FileFinder = new CFileFinder();
	FileFinder->ConstructL(CEikonEnv::Static()->FsSession(), strlist);
	FileFinder->StartFind();

	//CActiveSchedulerWait *w = new CActiveSchedulerWait();
	//w->Start();
	CActiveScheduler::Start();
	RTimer heartbeat;
	heartbeat.CreateLocal();
	TRequestStatus heartbeatStatus;
	heartbeat.After(heartbeatStatus,1000000);
	do {
		User::WaitForAnyRequest();
		if (FileFinder->IsDone() || iSearchDone) {
			heartbeat.Cancel();
			break;
		}
		if (heartbeatStatus!=KRequestPending) {
			heartbeat.After(heartbeatStatus,1000000);
		}
	} while (!FileFinder->IsDone() && !iSearchDone);
	if (iSearchDone) { //canceled
		CActiveScheduler::Stop();
	}
	if (!iDocument->Model()->IsReady() && strlist->Count()>0){
		if (iDocument->Model()->ChangeBible((*(strlist->Content()))[0])) {
			SaveSettingsL();
                        iDocument->Model()->FillVerse();
		}
	}

	heartbeat.Close();
	delete FileFinder;
	strlist->SaveToFileL(iBibleList);
	CleanupStack::PopAndDestroy(strlist);
	/* despite of the name (does NOT ends with a D),
	 * the docs says that this deletes the dialog
	 */
	dlg->ProcessFinishedL();
	
/*this one is to slow*/
#if 0
	CStringList *strlist =CStringList::NewLC(CEikonEnv::Static()->FsSession());
	CAknWaitNoteWrapper* waitNoteWrapper = CAknWaitNoteWrapper::NewL();
	CleanupStack::PushL(reinterpret_cast<CBase*>(waitNoteWrapper));
	if (iFinderEngine!=NULL) {
		delete iFinderEngine;
	}
	iFinderEngine = new (ELeave) CFileFinderEngine(CEikonEnv::Static()->FsSession(), strlist);
	iFinderEngine->ConstructL();
	iFinderEngine->InitFind();
	waitNoteWrapper->ExecuteL( R_FIND, *this, ETrue);	
	
	if (!iDocument->Model()->IsReady() && strlist->Count()>0){
		if (iDocument->Model()->ChangeBible((*(strlist->Content()))[0])) {
			SaveSettingsL();
			iDocument->Model()->FillVerse();
		}
	}
	delete iFinderEngine;
	iFinderEngine = NULL;
	strlist->SaveToFileL(iBibleList);
	CleanupStack::PopAndDestroy(waitNoteWrapper);
	CleanupStack::PopAndDestroy(strlist);
#endif
}

void CS60BibleMainView::GotoVerse()
{
	TBuf<256> textData(NULL);
	char buff[256];
	textData = iLastVerse;
	CAknTextQueryDialog *dlg = new (ELeave)CAknTextQueryDialog(textData,
		CAknQueryDialog::ENoTone);
		
	if (!(dlg->ExecuteLD(R_AKNEXQUERY_ONELINE_DATA_QUERY))) return;
	iDocument->Model()->MyDesToString(textData, buff);
	if (iDocument->Model()->SeekVerseAsString(1, buff)) {
		iDocument->Model()->ClearVerse();
		iDocument->Model()->FillVerse();
		iLastVerse = textData;
		TBool found = EFalse;
		for (int i = 0; i<iHistory->Count(); i++){
			if (((*(iHistory->Content()))[i])==textData) {
				found = ETrue;
			}
		}
		if (!found) iHistory->PrependL(textData);
		if (iHistory->Count()>10) {
			iHistory->Delete(10);
		}
		iHistory->SaveToFileL(iBibleHistory);
	} else {
		CAknNoteDialog* dlg = new CAknNoteDialog();
		dlg->ExecuteLD(R_VERSENOTFOUND);
	}
}

void CS60BibleMainView::ShowColorSelectionL()
{

	TInt index( 0 ); 
	CAknListQueryDialog* dlg = new( ELeave ) CAknListQueryDialog( &index ); 
	if ( dlg->ExecuteLD(R_COLOR_LIST) ) 
	{
		TRgb current = iDocument->Model()->GetColor((Viewer::ETextObject)index);	
		CColorDialog *d = new (ELeave) CColorDialog(&current);
		if (d->ExecuteLD(R_CHOOSE_COLOR)) {
			iDocument->Model()->SetColor((Viewer::ETextObject)index, current);
		}
	}
}


void CS60BibleMainView::HandleCommandL(TInt aCommand)
{
	switch(aCommand) {
        case EAknSoftkeyExit:
			{
				SaveSettingsL();
				AppUi()->HandleCommandL(aCommand);
				break;
			}
        case KSelectBible:
			{
        			ShowBibleSelection();
				break;
        		}
        case KSelectEncoding:
                        {
                               ShowEncodingSelection();
                               break;
                        }
	case KSelectFont:
			{
				ShowFontSelection();
				break;
			}
        case KSearch:
			{
				SearchForBibleFilesL();
				break;
			}
        case KGoto:
			{
				GotoVerse();
				break;
			}
        case KLanguage:
			{
				ChooseLanguageL();
				break;
			}

        case KFullscreen:
			{
				iContainer->ToggleFullScreen();
				iFullScreen = !iFullScreen;
				break;
			}
        case KShowHistory:
			{
				ShowHistory();
				break;
			}
        case KCopy:
			{
				if (iDocument->Model()->IsReady()){
					iDocument->Model()->CopyToClipboardL(1);
					CAknInformationNote* informationNote =
						new (ELeave)CAknInformationNote;

					HBufC* str = CCoeEnv::Static()->AllocReadResourceLC(R_S60BIBLE_STRING_TEXTCOPIED);
					informationNote->ExecuteLD(*str);
					CleanupStack::PopAndDestroy(str);
				}
				break;
			}
	/*navigation*/
	case KPrevLine:
			{
				iDocument->Model()->ScrollUp(1);
				iDocument->Model()->UpdateBuffer();
				iContainer->DrawDeferred();
				break;
			}
	case KNextLine:
			{
				iDocument->Model()->ScrollDown(1);
				iDocument->Model()->UpdateBuffer();
				iContainer->DrawDeferred();
				break;
			}
	case KPrevPage:
			{
				iDocument->Model()->PageUp();
				iContainer->DrawDeferred();
				break;
			}
	case KNextPage:
			{
				iDocument->Model()->PageDown();
				iContainer->DrawDeferred();
				break;
			}
	case KNextVerse:
			{
				iDocument->Model()->NextVerse();
				iContainer->DrawDeferred();
				break;
			}
	case KPrevVerse:
			{
				iDocument->Model()->PrevVerse();
				iContainer->DrawDeferred();
				break;
			}
	case KNextChapter:
			{
				iDocument->Model()->NextChapter();
				iContainer->DrawDeferred();
				break;
			}
	case KPrevChapter:
			{
				iDocument->Model()->PrevChapter();
				iContainer->DrawDeferred();
				break;
			}
	case KGotoTop:
			{
				iDocument->Model()->GotoTop();
				iContainer->DrawDeferred();
				break;
			}
	case KGotoBottom:
			{
				iDocument->Model()->GotoBottom();
				iContainer->DrawDeferred();
				break;
			}
	case KNextBook:
			{
				iDocument->Model()->NextBook();
				iContainer->DrawDeferred();
				break;
			}
	case KPrevBook:
			{
				iDocument->Model()->PrevBook();
				iContainer->DrawDeferred();
				break;
			}
	case KBookList:
			{
				if (iDocument->Model()->IsReady()){
					iContainer->SetFullScreenL(EFalse);
					AppUi()->ActivateLocalViewL(TUid::Uid(KBookListViewId));
				}
				break;
			}
	case KSettings:
			{
				iContainer->SetFullScreenL(EFalse);
				AppUi()->ActivateLocalViewL(TUid::Uid(KSettingListViewID));
				break;
			}
	case KKeyBinding:
			{
				iContainer->SetFullScreenL(EFalse);
				AppUi()->ActivateLocalViewL(TUid::Uid(KKeyBindingViewID));
				break;
			}
        case KColor:
			{
				ShowColorSelectionL();
				break;
			}

        case KAbout:
			{
				iContainer->SetFullScreenL(EFalse);
				AppUi()->ActivateLocalViewL(TUid::Uid(KAboutViewID));
				break;
			}
	case KAddToBookmark:
			{
				iDocument->Model()->AddToBookmark();
				iDocument->Model()->SaveBookmarkL();
				CAknInformationNote* informationNote = new (ELeave) CAknInformationNote;
				HBufC* str = CCoeEnv::Static()->AllocReadResourceLC(R_S60BIBLE_STRING_ADDED_TO_BOOKMARK);
				informationNote->ExecuteLD(*str);
				CleanupStack::PopAndDestroy(str);
				break;
			}
    case KGotoBookmark:
			{
				iContainer->SetFullScreenL(EFalse);
				AppUi()->ActivateLocalViewL(TUid::Uid(KBookmarkViewID));
				break;
			}
    case KZoomIn:
			{
				iDocument->Model()->ZoomIn();
				iContainer->DrawDeferred();
				break;
			}
    case KZoomOut:
			{
				iDocument->Model()->ZoomOut();
				iContainer->DrawDeferred();			
				break;
			}			
	default:
		{
			User::Panic(KS60BiblePanicName, ES60BibleUnknownCommand);
			break;
		}
	}
}


void CS60BibleMainView::ChooseLanguageL()
{
    HBufC* appName = iDocument->Application()->AppFullName().AllocLC();
    TParsePtrC parse(*appName);

    // Construct search path
    _LIT(KExtra, "S60Bible.r??");
    TFileName file;
    file.Copy(parse.DriveAndPath());
    file.Append(KExtra);

    RFs& fs = iCoeEnv->FsSession();

    CDir* entries;
    fs.GetDir(file, KEntryAttNormal, ESortByName, entries);
    CleanupStack::PushL(entries);

    CDesCArray* list = new(ELeave) CDesCArrayFlat(5);
    CleanupStack::PushL(list);

    // Fill up listbox
    for (TInt i = 0; i < entries->Count(); ++i)
    {
        _LIT(KFormat, "\t%S");
        file.Format(KFormat, &(*entries)[i].iName);
        list->AppendL(file);
    }

    TInt index;
    CAknSelectionListDialog* dialog = CAknSelectionListDialog::NewL(index, list, 0);
    if (dialog->ExecuteLD(R_LIST_DIALOG))
    {
        file.Copy(parse.DriveAndPath());
        file.Append((*entries)[index].iName);

        // Replace resource file
        if (iOffset) {
			Log(_L("Deleting resourcefile"));
            iCoeEnv->DeleteResourceFile(iOffset);
		}
		Log(_L("Replacing resourcefile"));
		Log(file);
		iOffset = iCoeEnv->AddResourceFileL(file);
        // Load cba, menu, etc as necessary
        //iEikonEnv->AppUiFactory()->Cba()->SetCommandSetL(R_SOFTKEYS);
		Log(_L("Replacing Menu title"));
        iEikonEnv->AppUiFactory()->MenuBar()->SetMenuTitleResourceId(R_S60BIBLE_MAINMENU);
    }
		Log(_L("Done Replacing Menu title"));
    CleanupStack::PopAndDestroy(3, appName);    // list, entries, appName
	Log(_L("CleanUp stack OK"));
}
